/*
 * (c) 2010, SWD Embedded Systems Limited, http://www.kpda.ru
 */

// Module Description:  header for HID class driver

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <share.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <termios.h>
#include <sys/io-hid.h>
#include <sys/usbdi.h>

#define UMASS_DEREG_PRIORITY 8

/* EGALAX */
#define USB_VENDOR                         0x0eef
#define USB_DEVICE                         0x0001

#define TOUCH_CONTROL_EP                   1
#define TOUCH_INTIN_EP                     2

#define HIDD_BUTTON_LEFT                   0x01
#define HIDD_BUTTON_RIGHT                  0x02

//                          HID STATUS DEFINITION
#define USB_DEVICE_IN_USE                  0x01


#define TOUCH_CONTROL_EP                          1
#define TOUCH_INTIN_EP                            2

#define SYNC_BIT            0x80
#define PLAYER_ID_SET       0x20
#define PRESSURE_SET        0x40

/* Request Packet Information */
#define HEADER                                    0x0A
#define CMD_LENGTH                                0x01

/* FIRMWARE Commands */
#define CHECK_ACTIVE                              0x41	/* Check to see if the controller is attached (loopback) */
#define REQUEST_FIRMWARE_REVISION                 0x44  /* Request Firmware Version */
#define REQUEST_CONTROLLER_INTERFACE              0x45	/* Request Controller Interface, responses are either USB or Serial */
#define REQUEST_CONTROLLER_TYPE                   0x46  /* Request Controller type, will return string information on the controller */
#define CALIBRATION_PARAMETERS                    0x37
#define SET_RAW_DATA                              0x37
#define CALCULATE_CALIB_PARAMETERS_DATA           0xFF

#define SET_REPORT_MODE                           0x34	/* Set Report Mode, this will change the mode between raw and calibrated points */
#define RAW_MODE                                  0x00
#define CALIBRATED_MODE                           0x01

#define SET_SENSITIVITY                           0x42
#define SET_SENSITIVITY_REPLY                     0x58

#define TOGGLE_FILTERS                            0x36
#define TOGGLE_FILTERS_DATA                       0x0E
#define FILTERS_ENABLED                           0x00
#define FILTERS_DISABLED                          0x01

#define RESET_CONTROLLER                          0x79

/* FIRMWARE RESPONSESE */
#define SET_REPORT_MODE_ACK                       0x33


#define MINIMUM_FIRMWARE_REPLY_LENGTH             3
#define RING_BUFFER_SIZE                          256

/* Flags */
#define PEN_UP                                    0x00
#define PEN_DOWN                                  0x01
#define TOUCH_UNBLOCK                             2
#define TOUCH_PRESENT                             4
#define TOUCH_IN_SHUTDOWN                         8
#define FLAG_NO_INIT        											0x10
#define FLAG_GET_CONTROLLER_INFO                  0x20

/* ID Report */
#define ID_REPORT_ID                              0x0C


typedef struct _report_buf {
	_uint16		bytes_left;
	_uint8		*report_data;
} report_buffer;

typedef struct hid_descriptor {
    _uint8 			bLength;
    _uint8			bDescriptorType;
    _uint16			bcdHID;
    _uint8			bCountryCode;
    _uint8			bNumDescriptors;
    _uint8			bReportDescriptorType;
    _uint16			wDescriptorLength;
} hid_descriptor_t;

typedef struct _usb {
	TAILQ_ENTRY(_usb)           dlink;
	usbd_device_instance_t			usbd_instance;
	struct usbd_connection			*connection;
	int                         hid_hdl;
	void                        *hiddi_reg; // handle for use with hiddi
	struct usbd_device          *device;
	_uint32                     status;
	_uint16                     idle_rate;
	_uint16                     protocol;
	struct usbd_pipe            *ep_cntl;
	struct usbd_pipe            *ep_int_in;
	struct usbd_pipe            *ep_int_out;
	_uint32                     ep_int_in_size;
	_uint32                     ep_int_out_size;
	_uint8                      *ep_int_buf;
	struct usbd_urb             *urb;
	_uint32                     report_len;
	void                        *report_data; //report_descriptor data
	void                        *rbuffer;
	int                         coid;
	_uint8                      ring_buffer[256];
	int                         ring_start, ring_end, ring_buffer_current_size;
} usb_t;

typedef struct _usb_ctrl {
	TAILQ_HEAD(,_usb)   dlist;		// linked list of hid devices
	_uint32             cflags;
	_uint32             verbose;
	_uint8              sensitivity;
	_uint16             delay;
	_uint32             devno;
	_uint32             ndevs;		// number of hid devices
	_uint8              *path;
	void                *dll_hdl;	
	io_hid_self_t       *ioh;	
	struct usbd_connection			*connection[5];
	_uint16             num_conn;	
	_uint8              rsvd[2];	
//	_uint8              controller_command;
	_uint8              partial_reply[255];
	int                 partial_reply_expected_len;
	int                 partial_reply_current_len;
	int                 sending_calibration_data;
} usb_ctrl_t;

typedef struct _egalax_data_packet {
  _uint8 cmd;
  _uint8 num_points;
	_uint8 index;
  _uint16 x, y;
} egalax_data_packet_t;


int	NumCtl = 0;
usb_ctrl_t UsbCtrl[5];
