/*
 * Copyright 2017, CBD BC
 * 
 * $QNXLicenseC:
 * Copyright 2014, QNX Software Systems.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not reproduce, modify or distribute this software except in
 * compliance with the License. You may obtain a copy of the License
 * at: http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as
 * contributors under the License or as licensors under other terms.
 * Please review this entire file for other proprietary rights or license
 * notices, as well as the QNX Development Suite License Guide at
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */



#ifndef _GPIO_AM35_H_INCLUDED
#define _GPIO_AM35_H_INCLUDED

#define GPIO_DEVICE_NAME	"/dev/gpio"

#define GPIO_INPUT					0
#define GPIO_OUTPUT					1

#define GPIO_INT_LEVEL_UNCHANGED	0
#define GPIO_INT_LEVEL_LOW			1
#define GPIO_INT_LEVEL_HIGH			2

#define GPIO_RESMGR_NPARTS_MIN		2
#define GPIO_RESMGR_MSGSIZE_MIN		16
#define GPIO_CLIENTS_MAX			32

typedef struct {
	uint8_t		pin_num;
} gpio_devctl_input_t;

typedef struct {
	uint8_t		pin_num;
} gpio_devctl_output_t;

typedef struct {
	uint8_t		pin_num;
	uint8_t		data;
} gpio_devctl_read_t;

typedef struct {
	uint8_t		pin_num;
	uint8_t		data;
} gpio_devctl_write_t;

typedef struct {
	uint8_t		pin_num;
	uint8_t		level;
	int			coid;
} gpio_devctl_int_t;

// A union of all devctl messages
typedef union {
		gpio_devctl_input_t			set_input;
		gpio_devctl_output_t		set_output;
		gpio_devctl_read_t			cmd_read;
		gpio_devctl_write_t			cmd_write;
		gpio_devctl_int_t			cmd_intterupt;
} gpio_devctl_t;

typedef struct {
	uint8_t		pin;
	uint8_t		level;
} gpio_interrupt_t;


#define GPIO_SET_INPUT				0
#define GPIO_SET_OUTPUT				1
#define GPIO_READ					2
#define GPIO_WRITE					3
#define GPIO_INT_ENABLE				4
#define GPIO_INT_DISABLE			5
#define GPIO_INT_WAIT_PULSE			6

#define DCMD_SET_INPUT				__DIOT(_DCMD_MISC, GPIO_SET_INPUT, gpio_devctl_input_t)
#define DCMD_SET_OUTPUT				__DIOT(_DCMD_MISC, GPIO_SET_OUTPUT, gpio_devctl_output_t)
#define DCMD_READ					__DIOTF(_DCMD_MISC, GPIO_READ, gpio_devctl_read_t)
#define DCMD_WRITE					__DIOTF(_DCMD_MISC, GPIO_WRITE, gpio_devctl_write_t)

#define DCMD_INT_ENABLE				__DIOTF(_DCMD_MISC, GPIO_INT_ENABLE, gpio_devctl_int_t)
#define DCMD_INT_DISABLE			__DIOTF(_DCMD_MISC, GPIO_INT_DISABLE, gpio_devctl_int_t)
#define DCMD_INT_ENABLE_WAIT_PULSE	__DIOTF(_DCMD_MISC, GPIO_INT_WAIT_PULSE, gpio_devctl_int_t)

#endif
