/***************************************************************************
 *                                                                         
 * Copyright 2017, CBD BC
 * 
 * GPIO hardware keyboard driver                                           
 *                                                                         
 ***************************************************************************/


#ifndef _GPIO_KBD_H
#define _GPIO_KBD_H

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <share.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <termios.h>
#include <sys/io-hid.h>
#include <fcntl.h>

#include <sys/types.h>
#include <devctl.h>

#include "gpio_am35.h"

#define POLL_INTERVAL           0
#define VERBOSE                 1
#define KEY_1_MODIFIES          2
#define KEY_1_SCAN_CODE         3
#define KEY_2_MODIFIES          4
#define KEY_2_SCAN_CODE         5
#define KEY_3_MODIFIES          6
#define KEY_3_SCAN_CODE         7
#define KEY_4_MODIFIES          8
#define KEY_4_SCAN_CODE         9
#define KEY_5_MODIFIES          10
#define KEY_5_SCAN_CODE         11
#define KEY_6_MODIFIES          12
#define KEY_6_SCAN_CODE         13

#define GPIOKBD_VENDOR_ID       (0)
#define GPIOKBD_DEVICE_ID       (0)
#define GPIOKBD_VERSION_ID      (0x100)

#define NUM_OF_INPUT_KEYS       (6)
#define DEFAULT_POLLING_INT     50

#define GPIO_ROW_1              177
#define GPIO_ROW_2              119
#define GPIO_ROW_3              182
#define GPIO_LINE_1             176
#define GPIO_LINE_2             178

struct _gpio_key {
    int                 gpio_isr;
    int                 iid;
    int                 pressed;
    int                 gpio_mask;
    uint64_t            lcc;
    const char          *name;
    int                 scan_code;
    int                 modifies;
};

typedef struct _gpiokbd_ctrl {
    _uint32             polling_interval;
    _uint32             verbose;
    void                *dll_hdl;
    io_hid_self_t       *ioh;
    int                 hid_hdl;
    struct _gpio_key    key[6];
} gpiokbd_ctrl_t;

typedef struct _keybInputReport {
    _uint8              modifies;                       // State of modifiers (from U224 to U231)
    _uint8              reserv;                         // Reserved byte
    _uint8              pressKeys[NUM_OF_INPUT_KEYS];   // Usages from U0 to U255
} keybInputReport_t, * pKeybInputReport_t;

#endif /* _GPIO_KBD_H */
